import requests
import base64
from datetime import datetime, timedelta

class ValditIdentityServer:
    # Declaring class properties
    _productionUri = "https://www.valdit.com/identity"
    _sandBoxUri = "https://developer.valdit.com/identity"
    _stagingUri = "https://api.staging.valdit.com/identity"
    _get_token_url = False
    oauth_token = False
    expires_at = datetime.now()


    def __init__(self, user_id, user_secret, scope, endpointType = "production"):
        # Evnvironment switch
        if endpointType == "sandbox":
            self._idSvrUrl = self._sandBoxUri
        elif endpointType == "staging":
            self._idSvrUrl = self._stagingUri
        else:
            self._idSvrUrl = self._productionUri

        if user_id is False or user_secret is False or scope is False:
            raise ValueError('Authentication token must be in `query` or `header`')

        self.user_id = user_id
        self.user_secret = user_secret
        self.scope = scope

    # Get token method
    def validateToken(self):
        if self.oauth_token is False:
            self.getOauthToken()

        if self.expires_at < datetime.now():
            self.getOauthToken()

        return self.oauth_token        

    # Cretae token if it dosnt exist or it is expired
    def getOauthToken(self):
        if self._get_token_url is False:
            self.openIdConfiguration()

        encode_credentials = "Basic " + base64.b64encode(self.user_id + ":" + self.user_secret)
            
        headers = {'Content-Type': 'application/x-www-form-urlencoded', 'Authorization': encode_credentials}
        data = {'grant_type': 'client_credentials', 'scope': self.scope}
        
        # temp hardcoded url
        self._get_token_url = "https://developer.valdit.com/identity/connect/token"
        result = requests.post(self._get_token_url, headers=headers, data=data)
        
        if result.status_code is 200:
            result = result.json()
            self.oauth_token = result["access_token"]
            self.expires_at = datetime.now() + timedelta(seconds=result["expires_in"])

    # Get endpoint for generate token API
    def openIdConfiguration(self):
        # Building URL for API request
        uri_builder = self._idSvrUrl
        uri_builder += "/.well-known/openid-configuration"
        
        # Should user agent contain csharp?
        headers = {'User-agent': 'Valdit-Api-Client-Common-Client/1.0.0/csharp', 'Content-type': 'application/json'}

        response = requests.get(uri_builder, headers=headers);

        if response.status_code is 200:
            self._get_token_url = response.json()["token_endpoint"]
